<?php

namespace App\Services\File\BIM\BlackHole;

use App\Models\Enums\BimFileConvertStatus;
use App\Services\File\BIM\Abstracts\BIMAbstract;
use App\Services\File\BIM\BIMDriverEnum;
use App\Services\File\BIM\Contacts\BIMContact;
use GuzzleHttp\Psr7;
use Illuminate\Http\UploadedFile;

class BlackHole extends BIMAbstract
{
    public function uploadFile(UploadedFile $file, array $params = [])
    {
        $projectId = config("bim.black_hole.project_id");
        $treeInfo = $this->getTreeById($projectId);

        $chunkSize = 1024 * 1024 * 10; // 10M
        $chunkCount = (int)ceil($file->getSize() / $chunkSize);

        $modelFile = $this->modelFileCreate($projectId, $treeInfo['subNodes'][0]['nodeId'], $file, $chunkSize, $chunkCount);
        $dataSetId = $modelFile['dataSetId'];

        $fileResource = fopen($file, 'r');
        for ($i = 0; $i < $chunkCount; $i++) {
            $chunk = fread($fileResource, $chunkSize);

            $stream = Psr7\Utils::streamFor($chunk);

            $uploadFormData = [
                ['name' => 'FileId', 'contents' => $modelFile['fileId']],
                ['name' => 'FileInfo', 'contents' => $stream, 'filename' => $file->getClientOriginalName()],
                ['name' => 'FileDataId', 'contents' => $modelFile['fileDataId']],
                ['name' => 'Key', 'contents' => $modelFile['extraProperties']['ResumableKey']],
                ['name' => 'BlobIndex', 'contents' => $i],
            ];

            Client::getInstance()->post("/blackHole3D/project/modelFile/append", [
                'multipart' => $uploadFormData,
                'headers' => [
                    'fileUploadScheme' => 'Bim',
                    'fileUploadMode' => 'ResumableAppend',
                ],
            ]);
        }

        return [
            'bim_data_set_id' => $dataSetId,
            'bim_file_id' => $modelFile['fileId'],
            'bim_convert_status' => 0,
            'bim_driver' => BIMDriverEnum::BLACK_HOLE->value,
        ];
    }

    public function downloadSourceFile()
    {

    }

    public function viewDataSetModel(array $dataSetIDS)
    {
        $result = Client::getInstance()->post("/blackHole3D/project/dataSet/viewDataSetModel", [
            'json' => [
                'dataSetIds' => $dataSetIDS
            ]
        ]);

        return $result['data'] ?? [];
    }

    public function findConvertStatus(string $dataSetId): array
    {
        $result = Client::getInstance()->post("/blackHole3D/project/dataSet/getConvertStatus", [
            'json' => [
                'dataSetId' => $dataSetId
            ]
        ]);

        $status = match ($result['data']['status']) {
            0, 1 => BimFileConvertStatus::IN_QUEUE->value,
            2 => BimFileConvertStatus::CONVERTING->value,
            5 => BimFileConvertStatus::PAUSE->value,
            10 => BimFileConvertStatus::DONE->value,
            20 => BimFileConvertStatus::FAILED_TO_ADD_QUEUE->value,
        };

        return $this->convertStatusFormat($status, $result['data']['status']);
    }

    public function addToConvertQueue(string $dataSetId): array
    {
        $formData = [
            'dataSetId' => $dataSetId,
            'execConvertingCallback' => '',
            'isBuildDbIndex' => true,
            'isResetConverting' => true,
            'paramFilter' => new \stdClass(),
            'fileNames' => [],
            'useDefaultConfig' => false,
        ];

        $result = Client::getInstance()->post("/blackHole3D/project/modelFile/addToConvertQueue", [
            'json' => $formData,
        ]);

        return $result['data'] ?? [];
    }


    /**
     * 模型文件上传准备
     *
     * @param string $projectId
     * @param string $nodeId
     * @param UploadedFile $file
     * @param int $chunkSize
     * @param int $chunkCount
     * @return array|mixed
     */
    protected function modelFileCreate(string $projectId, string $nodeId, UploadedFile $file, int $chunkSize, int $chunkCount)
    {
        $formData = [
            'ProjId' => $projectId,
            'DataSetId' => $nodeId,
            'FileName' => uniqid() . "_" . $file->getClientOriginalName(),
            'FileSize' => $file->getSize(),
            'FileType' => $file->getExtension() ? $file->getExtension() : pathinfo($file->getClientOriginalName(), PATHINFO_EXTENSION),
            'Scheme' => 'Bim',
            'BlobCount' => $chunkCount,
            'BlobSize' => $chunkSize,
            'UploadWay' => 0
        ];

        $result = Client::getInstance()->post("/blackHole3D/project/modelFile/create", [
            'form_params' => $formData,
            'headers' => [
                'fileUploadScheme' => 'Bim',
                'fileUploadMode' => 'ResumableCreate'
            ]
        ]);

        return $result['data'] ?? [];
    }

    protected function getTreeById(string $projectId)
    {
        $result = Client::getInstance()->post("/blackHole3D/project/modelTree/getTreeById", [
            'json' => [
                'projId' => $projectId
            ]
        ]);

        return $result['data'] ?? [];
    }

    protected function addLeafNodes(array $nodes, string $parentId = "")
    {
        $items = [];
        foreach ($nodes as $node) {
            $items[]['dataSetName'] = $node;
        }

        $result = Client::getInstance()->post("/blackHole3D/project/modelTree/addLevelNodes", [
            'json' => [
                'parentId' => $parentId,
                'dataSets' => $items
            ]
        ]);

        return $result['data'] ?? [];
    }
}
